import os
import random
import matplotlib.pyplot as plt


def getRandomColor():
    """获取一个随机的颜色"""
    colorArr = ['1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F']
    color = ""
    for i in range(6):
        color += colorArr[random.randint(0, 14)]
    return "#" + color


class GraphBase:
    """
    一个基于 matplotlib 实现的通用绘图接口
    """

    def __init__(self):
        # 记录所有可用的 Marker
        # https://matplotlib.org/stable/api/markers_api.html
        self.markers = ["o", "v", "^", "<", ">", "1", "2", "3", "4", "8", "s", "p", "P", "*", "h", "H", "+",
                        "x", "X", "D", "d", "|", "_", ".", ","]
        # 记录当前图已经使用的Marker
        self.currentUsedMarker = {}
        # 记录常用的 color
        self.colors = [
            "tab:blue", "tab:orange", "tab:green", "tab:red", "tab:purple",
            "tab:brown", "tab:pink", "tab:gray", "tab:olive", "tab:cyan", "red", "green", "blue", "c", "m", "y",
        ]
        # 记录当前图已经使用的 color
        self.currentUsedColor = {}

    def autoSelectMarker(self) -> str:
        """
        从所有的marker中自动选择 Marker，保证不和已有的Marker重复
        :return:
        """
        for marker in self.markers:
            if marker not in self.currentUsedMarker:
                return marker
        return ""

    def autoSelectColor(self) -> str:
        for color in self.colors:
            if color not in self.currentUsedColor:
                return color
        while True:
            randomColor = getRandomColor()
            if randomColor not in self.currentUsedColor:
                break
        return randomColor

    def xlim(self, *args, **kwargs):
        plt.xlim(*args, **kwargs)
        return self

    def ylim(self, *args, **kwargs):
        plt.ylim(*args, **kwargs)
        return self

    def xticks(self, ticks=None, labels=None, **kwargs):
        plt.xticks(ticks, labels, **kwargs)
        return self

    def title(self, label, fontdict=None, loc=None, pad=None, **kwargs):
        plt.title(label, fontdict=fontdict, loc=loc, pad=pad, **kwargs)
        return self

    def xlabel(self, xlabel, fontdict=None, labelpad=None, loc=None, **kwargs):
        plt.xlabel(xlabel, fontdict, labelpad, loc=loc, **kwargs)
        return self

    def ylabel(self, ylabel, fontdict=None, labelpad=None, loc=None, **kwargs):
        plt.ylabel(ylabel, fontdict, labelpad, loc=loc, **kwargs)
        return self

    def legend(self, *args, **kwargs):
        plt.legend(*args, **kwargs)
        return self

    def plot(self, x, y, *args,
             color: str = "&&",
             linewidth: float = 2, linestyle: str = "dotted", marker: str = "&&",
             markerfacecolor: str = "none", markersize: float = 6,
             **kwargs):
        if "linewidth" not in kwargs:
            kwargs["linewidth"] = linewidth
        if "linestyle" not in kwargs:
            kwargs["linestyle"] = linestyle
        # 默认自动选择Marker
        if marker == "&&":
            marker = self.autoSelectMarker()
            self.currentUsedMarker[marker] = marker
        if "marker" not in kwargs:
            kwargs["marker"] = marker
        if "markerfacecolor" not in kwargs:
            kwargs["markerfacecolor"] = markerfacecolor
        if "markersize" not in kwargs:
            kwargs["markersize"] = markersize
        # 默认自动选择 color
        if color == "&&":
            color = self.autoSelectColor()
            self.currentUsedColor[color] = color
        if "color" not in kwargs:
            kwargs["color"] = color
        plt.plot(x, y, *args, **kwargs)
        return self

    def show(self):
        """
        绘制吞吐量图，并show
        :return:
        """
        plt.show()
        return self

    def drawAndSave(self, dir: str, name: str):
        """
        绘制吞吐量图，并保存到图片当中
        :return:
        """
        if not os.path.exists(dir):
            os.makedirs(dir)
        plt.savefig(f"{dir}{os.sep}{name}")
        return self

    def close(self):
        plt.close()



from enum import Enum
from typing import Optional
from .common import GraphBase


class DropType(Enum):
    """
    丢包类型
    https://ndnsim.net/current/metric.html
    """
    Drop = 0


class DropTarget(Enum):
    """
    丢包目标
    https://ndnsim.net/current/metric.html
    """
    Packets = 0  # estimated rate (EWMA average) of packets within the last averaging period (number of packets/s)
    Kilobytes = 1  # estimated rate (EWMA average) within last averaging period (kilobytes/s)
    PacketRaw = 2  # absolute number of packets within last averaging period (number of packets).
    KilobytesRaw = 3  # absolute number of kilobytes transferred within the last averaging period (number of packets).


class DropItem:
    """
    一个用于描述 ndnsim 丢包采样结果的类
    """

    def __init__(self, Time: float, Node: str, Interface: str, Type: DropType, Packets: float,
                 Kilobytes: float, PacketsRaw: float, KilobytesRaw: float):
        self.Time = Time
        self.Node = Node
        self.Interface = Interface
        self.Type = Type
        self.Packets = Packets
        self.Kilobytes = Kilobytes
        self.PacketsRaw = PacketsRaw
        self.KilobytesRaw = KilobytesRaw

    def getValueByDropTarget(self, dropTarget: DropTarget):
        if dropTarget == DropTarget.Packets:
            return self.Packets
        elif dropTarget == DropTarget.Kilobytes:
            return self.Kilobytes
        elif dropTarget == DropTarget.PacketRaw:
            return self.PacketsRaw
        else:
            return self.KilobytesRaw

    @staticmethod
    def parseLine(line: str):
        # 0.5	C1	combined	Drop	0	0	0	0
        values = line.strip().split("\t")
        if len(values) < 8:
            return None
        return DropItem(
            float(values[0].strip()),
            values[1].strip(),
            values[2].strip(),
            DropType[values[3].strip()],
            float(values[4].strip()),
            float(values[5].strip()),
            float(values[6].strip()),
            float(values[7].strip()),
        )


class NodeFaceDrop:
    """
    一个用于描述某个接口丢包情况的类
    """

    def __init__(self, Node: str, Interface: str):
        self.Node = Node
        self.Interface = Interface
        self.typeMap = dict()  # 用于存储不同 Drop Type 的采样记录
        for dropType in DropType:
            self.typeMap[dropType.name] = []

    def appendItem(self, item: DropItem):
        """
        新增一条采样记录
        :param item:
        :return:
        """
        # 如果不是当前接口的统计记录，忽略
        if item.Node != self.Node or item.Interface != self.Interface:
            return
        self.typeMap[item.Type.name].append(item)

    def getX(self, dropType: DropType = DropType.Drop, samplingInterval: float = 1.0):
        """
        获取采样时间列表
        :param dropType:
        :param samplingInterval:
        :return:
        """
        lastCount, res = 0, []
        for item in self.typeMap[dropType.name]:
            if int(item.Time / samplingInterval) != lastCount:
                lastCount = int(item.Time / samplingInterval)
            else:
                continue
            res.append(lastCount * samplingInterval)
        return res

    def getY(self, dropType: DropType = DropType.Drop,
             dropTarget: DropTarget = DropTarget.PacketRaw,
             samplingInterval: float = 1.0):
        """
        获取丢包数统计列表
        :param dropType:
        :param dropTarget:
        :param samplingInterval:
        :return:
        """
        lastCount, res, currentDropPacketsNum = 0, [], 0
        for item in self.typeMap[dropType.name]:
            if int(item.Time / samplingInterval) != lastCount:
                lastCount = int(item.Time / samplingInterval)
                currentDropPacketsNum += item.getValueByDropTarget(dropTarget)
                res.append(currentDropPacketsNum)
                currentDropPacketsNum = 0
            else:
                currentDropPacketsNum += item.getValueByDropTarget(dropTarget)
                continue
        return res


class NodeDrop:
    """
    一个用于描述节点各个接口丢包的类
    """

    def __init__(self, Node: str):
        self.interfaceMap = dict()
        self.Node = Node

    def appendItem(self, item: DropItem):
        """
        新增一条采样记录
        :param item:
        :return:
        """
        # 如果不是当前节点的统计记录，忽略
        if item.Node != self.Node:
            return
        if item.Interface not in self.interfaceMap:
            self.interfaceMap[item.Interface] = NodeFaceDrop(item.Node, item.Interface)
        self.interfaceMap[item.Interface].appendItem(item)

    def getX(self, interface: str = "combined", dropType: DropType = DropType.Drop,
             samplingInterval: float = 1.0) -> [float]:
        """
        获取采样时间列表
        :param interface:
        :param dropType:
        :param samplingInterval:
        :return:
        """
        if interface not in self.interfaceMap:
            return []
        return self.interfaceMap[interface].getX(dropType, samplingInterval)

    def getY(self, interface: str, dropType: DropType = DropType.Drop, dropTarget: DropTarget = DropTarget.PacketRaw,
             samplingInterval: float = 1.0) -> [float]:
        """
        获取丢包数统计列表
        :param interface:
        :param dropType:
        :param dropTarget:
        :param samplingInterval:
        :return:
        """
        if interface not in self.interfaceMap:
            return []
        return self.interfaceMap[interface].getY(dropType, dropTarget, samplingInterval)


class Drop:
    """
    一个用于解析 ndnsim 丢包统计结果的类
    """

    def __init__(self):
        self.nodeMap = dict()

    def getByNode(self, node: str) -> Optional[NodeDrop]:
        if node not in self.nodeMap:
            return None
        return self.nodeMap[node]

    @staticmethod
    def parse(inputFile: str):
        """
        传入一个 ndnsim 丢包统计结果文件，从中解析出丢包数据
        :param inputFile:
        :return:
        """
        drop = Drop()
        with open(inputFile, "r") as f:
            # 首先忽略表头
            # Time	Node	Interface	Type	Packets	Kilobytes	PacketsRaw	KilobytesRaw
            f.readline()

            # 从第一行开始解析
            line = f.readline()
            while line:
                if line.strip() != "":
                    item = DropItem.parseLine(line)
                    if item:
                        if item.Node not in drop.nodeMap:
                            drop.nodeMap[item.Node] = NodeDrop(item.Node)
                        drop.nodeMap[item.Node].appendItem(item)
                line = f.readline()
        return drop


class DropGraph(GraphBase):
    """
    一个用于实现绘制丢包数图的类
    """

    def __init__(self, drop: Drop):
        super().__init__()
        self.drop = drop
        self.samplingInterval = 1.0
        self.dropType = DropType.Drop
        self.dropTarget = DropTarget.PacketRaw

    def setSamplingInterval(self, samplingInterval: float):
        """
        设置采样间隔 => 如果 samplingInterval = 1.0 表示绘图时，每秒采样一次
        :param samplingInterval:
        :return:
        """
        self.samplingInterval = samplingInterval
        return self

    def setDropType(self, dropType: DropType):
        """
        设置丢包类型
        :param dropType:
        :return:
        """
        self.dropType = dropType
        return self

    def setDropTarget(self, dropTarget: DropTarget):
        """
        设置丢包目标
        :param dropTarget:
        :return:
        """
        self.dropTarget = dropTarget
        return self

    def plot(self, node: str, *args,
             interface: str = "combined",
             color: str = "&&",
             linewidth: float = 2, linestyle: str = "dotted", marker: str = "&&",
             markerfacecolor: str = "none", markersize: float = 6,
             **kwargs):
        node = self.drop.getByNode(node)
        if not node:
            print("not exist node: ", node)
            return
        if "label" not in kwargs:
            kwargs["label"] = node.Node
        x, y = node.getX(interface, samplingInterval=self.samplingInterval), \
               node.getY(interface, self.dropType, self.dropTarget, self.samplingInterval)
        super().plot(x, y, *args, **kwargs)
        return self

    @staticmethod
    def parse(inputFile: str):
        return DropGraph(Drop.parse(inputFile))



from enum import Enum
from typing import Optional
from .common import GraphBase


class DelayType(Enum):
    """
    延迟类型
    https://ndnsim.net/current/metric.html
    """
    # LastDelay means that DelayS and DelayUS represent delay between last Interest sent and Data packet received
    # LastDelay意味着DelayS和DelayUS代表最后发送的兴趣和接收的数据包之间的延迟
    LastDelay = 0
    # FullDelay means that DelayS and DelayUS represent delay between first Interest sent and Data packet received
    # (i.e., includes time of Interest retransmissions)
    # FullDelay是指DelayS和DelayUS代表发送的第一个感兴趣的数据包和接收的数据包之间的延迟
    FullDelay = 0


class DelayTarget(Enum):
    """
    延迟目标
    """
    DelayS = 0  # 按秒统计的延迟
    DelayMS = 1  # 按毫秒统计的延迟
    DelayUS = 2  # 按微秒统计的延迟


class DelayItem:
    """
    一个用于描述 ndnsim 延迟采样结果的类
    https://ndnsim.net/current/metric.html#application-level-trace-helper
    """

    def __init__(self, Time: float, Node: str, AppId: int, SeqNo: int, Type: DelayType, DelayS: float,
                 DelayUS: float, RetxCount: int, HopCount: int):
        """
        :param Time:
        :param Node:
        :param AppId:
        :param SeqNo:
        :param Type:
        :param DelayS:
        :param DelayUS:
        :param RetxCount:
        :param HopCount:
        """
        self.Time = Time
        self.Node = Node
        self.AppId = AppId
        self.SeqNo = SeqNo
        self.Type = Type
        self.DelayS = DelayS
        self.DelayUS = DelayUS
        self.RetxCount = RetxCount
        self.HopCount = HopCount

    def getValueByDelayTarget(self, delayTarget: DelayTarget) -> float:
        if delayTarget == DelayTarget.DelayS:
            return self.DelayS
        elif delayTarget == DelayTarget.DelayMS:
            return self.DelayUS / 1000.0
        else:
            return self.DelayUS

    @staticmethod
    def parseLine(line: str):
        # 0.14809	C2	0	1	LastDelay	0.0610684	61068.4	1	3
        values = line.strip().split("\t")
        if len(values) < 9:
            return None
        return DelayItem(
            float(values[0].strip()),
            values[1].strip(),
            int(values[2].strip()),
            int(values[3].strip()),
            DelayType[values[4].strip()],
            float(values[5].strip()),
            float(values[6].strip()),
            int(values[7].strip()),
            int(values[8].strip())
        )


class NodeAppDelay:
    """
    用于描述某个节点上某个App的延迟 => 使用 AppId 来区分同一个节点上的不同App
    """

    def __init__(self, Node: str, AppId: int):
        self.Node = Node
        self.AppId = AppId
        self.typeMap = dict()  # 用于存储不同 Delay Type 的采样记录
        for delayType in DelayType:
            self.typeMap[delayType.name] = []

    def appendItem(self, item: DelayItem):
        """
        新增一条采样记录
        :param item:
        :return:
        """
        # 如果不是当前App的统计记录，忽略
        if item.Node != self.Node or item.AppId != self.AppId:
            return
        self.typeMap[item.Type.name].append(item)

    def getX(self, delayType: DelayType = DelayType.LastDelay, samplingInterval: float = 1.0):
        """
        获取采样时间列表
        :param delayType:
        :param samplingInterval:
        :return:
        """
        lastCount, res = 0, []
        for item in self.typeMap[delayType.name]:
            if int(item.Time / samplingInterval) != lastCount:
                lastCount = int(item.Time / samplingInterval)
            else:
                continue
            res.append(lastCount * samplingInterval)
        return res

    def getY(self, delayType: DelayType = DelayType.LastDelay,
             delayTarget: DelayTarget = DelayTarget.DelayMS,
             samplingInterval: float = 1.0):
        """
        获取延迟采样列表
        :param delayType:           延迟类型
        :param delayTarget:         延迟目标
        :param samplingInterval:    采样间隔
        :return:
        """
        lastCount, res = 0, []
        for item in self.typeMap[delayType.name]:
            if int(item.Time / samplingInterval) != lastCount:
                lastCount = int(item.Time / samplingInterval)
            else:
                continue
            res.append(item.getValueByDelayTarget(delayTarget))
        return res


class NodeDelay:
    """
    一个用于描述同一个节点上各个App延迟的类
    """

    def __init__(self, Node: str):
        self.Node = Node
        self.appMap = dict()

    def appendItem(self, item: DelayItem):
        """
        新增一条采样记录
        :param item:
        :return:
        """
        # 如果不是当前节点的采样记录，则忽略
        if item.Node != self.Node:
            return
        if item.AppId not in self.appMap:
            self.appMap[item.AppId] = NodeAppDelay(item.Node, item.AppId)
        self.appMap[item.AppId].appendItem(item)

    def getX(self, AppId: int, delayType: DelayType = DelayType.LastDelay, samplingInterval: float = 1.0):
        """
        获取某个App的采样时间列表
        :param AppId:
        :param delayType:
        :param samplingInterval:
        :return:
        """
        if AppId not in self.appMap:
            return []
        return self.appMap[AppId].getX(delayType, samplingInterval)

    def getY(self, AppId: int, delayType: DelayType = DelayType.LastDelay,
             delayTarget: DelayTarget = DelayTarget.DelayMS,
             samplingInterval: float = 1.0):
        """
        获取某个App的延迟采样列表
        :param AppId:
        :param delayType:
        :param delayTarget:
        :param samplingInterval:
        :return:
        """
        if AppId not in self.appMap:
            return []
        return self.appMap[AppId].getY(delayType, delayTarget, samplingInterval)


class Delay:
    """
    一个用于解析 ndnsim 延迟采样结果的类
    """

    def __init__(self):
        self.nodeMap = dict()

    def getByNode(self, node: str) -> Optional[NodeDelay]:
        if node not in self.nodeMap:
            return None
        return self.nodeMap[node]

    @staticmethod
    def parse(inputFile: str):
        """
        传入一个 ndnsim 采集的延迟采样结果文件，从中解析出延迟信息
        :param inputFile:
        :return:
        """
        delay = Delay()
        with open(inputFile, "r") as f:
            # 首先忽略表头
            # Time	Node	AppId	SeqNo	Type	DelayS	DelayUS	RetxCount	HopCount
            f.readline()

            # 从第一行开始解析
            line = f.readline()
            while line:
                if line.strip() != "":
                    item = DelayItem.parseLine(line)
                    if item:
                        if item.Node not in delay.nodeMap:
                            delay.nodeMap[item.Node] = NodeDelay(item.Node)
                        delay.nodeMap[item.Node].appendItem(item)
                line = f.readline()
        return delay


class DelayGraph(GraphBase):
    """
    一个用于实现绘制延迟图的类
    """

    def __init__(self, delay: Delay):
        super().__init__()
        self.delay = delay
        self.samplingInterval = 1.0
        self.delayType = DelayType.LastDelay
        self.delayTarget = DelayTarget.DelayMS

    def setSamplingInterval(self, samplingInterval: float):
        """
        设置采样间隔 => 如果 samplingInterval = 1.0 表示绘图时，每秒采样一次
        :param samplingInterval:
        :return:
        """
        self.samplingInterval = samplingInterval
        return self

    def setDelayType(self, delayType: DelayType):
        """
        设置延迟类型
        :param delayType:
        :return:
        """
        self.delayType = delayType
        return self

    def setDelayTarget(self, delayTarget: DelayTarget):
        """
        设置延迟目标
        :param delayTarget:
        :return:
        """
        self.delayTarget = delayTarget
        return self

    def plot(self, node: str, appId: int, *args,
             color: str = "&&",
             linewidth: float = 2, linestyle: str = "dotted", marker: str = "&&",
             markerfacecolor: str = "none", markersize: float = 6,
             **kwargs):
        node = self.delay.getByNode(node)
        if not node:
            print("not exist node: ", node)
            return
        if "label" not in kwargs:
            kwargs["label"] = node.Node
        x, y = node.getX(appId, samplingInterval=self.samplingInterval), \
               node.getY(appId, self.delayType, self.delayTarget, self.samplingInterval)
        super().plot(x, y, *args, **kwargs)
        return self

    @staticmethod
    def parse(inputFile: str):
        return DelayGraph(Delay.parse(inputFile))



from enum import Enum
from typing import Optional
from .common import GraphBase


class ThroughputType(Enum):
    """
    吞吐量类型
    https://ndnsim.net/current/metric.html
    """
    InInterests = 0
    OutInterests = 1
    InData = 2
    OutData = 3
    InNacks = 4
    OutNacks = 5
    InSatisfiedInterests = 6
    InTimedOutInterests = 7
    OutSatisfiedInterests = 8
    OutTimedOutInterests = 9

    # 下面的类型只在 FaceId = -1 有效
    SatisfiedInterests = 10
    TimedOutInterests = 11


class ThroughputTarget(Enum):
    """
    吞吐量目标
    https://ndnsim.net/current/metric.html
    """
    Packets = 0  # estimated rate (EWMA average) of packets within the last averaging period (number of packets/s)
    Kilobytes = 1  # estimated rate (EWMA average) within last averaging period (kilobytes/s)
    PacketRaw = 2  # absolute number of packets within last averaging period (number of packets).
    KilobytesRaw = 3  # absolute number of kilobytes transferred within the last averaging period (number of packets).


class ThroughputItem:
    """
    一个用于描述 ndnsim 吞吐量采样结果的类
    """

    def __init__(self, Time: float, Node: str, FaceId: int, FaceDescr: str, Type: ThroughputType,
                 Packets: float, Kilobytes: float, PacketRaw: float, KilobytesRaw: float):
        """
        :param Time:        时间，例如: 0.5
        :param Node:        节点名称，例如：C1
        :param FaceId:      Face id
        :param FaceDescr:   Face 描述
        :param Type:
        :param Packets:
        :param Kilobytes:
        :param PacketRaw:
        :param KilobytesRaw:
        """
        self.Time = Time  # simulation time
        self.Node = Node  # node id, globally unique
        self.FaceId = FaceId  # interface ID (-1 for combined metric)
        self.FaceDescr = FaceDescr
        self.Type = Type
        self.Packets = Packets
        self.Kilobytes = Kilobytes
        self.PacketRaw = PacketRaw
        self.KilobytesRaw = KilobytesRaw

    def getValueByThroughputTarget(self, throughputTarget: ThroughputTarget):
        if throughputTarget == ThroughputTarget.Packets:
            return self.Packets
        elif throughputTarget == ThroughputTarget.Kilobytes:
            return self.Kilobytes
        elif throughputTarget == ThroughputTarget.PacketRaw:
            return self.PacketRaw
        else:
            return self.KilobytesRaw

    @staticmethod
    def parseLine(line: str):
        # 0.5	C1	1	internal://	InInterests	0	0	0	0
        values = line.strip().split("\t")
        if len(values) < 9:
            return None
        return ThroughputItem(float(values[0].strip()),
                              values[1].strip(),
                              int(values[2].strip()),
                              values[3].strip(),
                              ThroughputType[values[4].strip()],
                              float(values[5].strip()),
                              float(values[6].strip()),
                              float(values[7].strip()),
                              float(values[8].strip()),
                              )


class NodeFaceThroughput:
    """
    一个用于描述节点某个接口吞吐量的类
    """

    def __init__(self, Node: str, FaceId: int):
        self.Node = Node
        self.FaceId = FaceId
        self.typeMap = dict()  # 用于存储不同 Throughput Type 的采样记录
        for throughputType in ThroughputType:
            self.typeMap[throughputType.name] = []

    def appendItem(self, item: ThroughputItem):
        """
        新增一条采样记录
        :param item:
        :return:
        """
        # 如果不是当前接口的统计记录，忽略
        if item.Node != self.Node or item.FaceId != self.FaceId:
            return
        self.typeMap[item.Type.name].append(item)

    def getX(self, throughputType: ThroughputType = ThroughputType.OutData, samplingInterval: float = 1.0):
        """
        获取采样时间列表
        :param throughputType:
        :param samplingInterval:
        :return:
        """
        lastCount, res = 0, []
        for item in self.typeMap[throughputType.name]:
            if int(item.Time / samplingInterval) != lastCount:
                lastCount = int(item.Time / samplingInterval)
            else:
                continue
            res.append(lastCount * samplingInterval)
        return res

    def getY(self, throughputType: ThroughputType = ThroughputType.OutData,
             throughputTarget: ThroughputTarget = ThroughputTarget.Kilobytes,
             samplingInterval: float = 1.0):
        """
        获取吞吐量列表
        :param throughputType:          吞吐量类型
        :param throughputTarget:        吞吐量目标
        :param samplingInterval:       采样间隔
        :return:
        """
        lastCount, res = 0, []
        for item in self.typeMap[throughputType.name]:
            if int(item.Time / samplingInterval) != lastCount:
                lastCount = int(item.Time / samplingInterval)
            else:
                continue
            res.append(item.getValueByThroughputTarget(throughputTarget))
        return res


class NodeThroughput:
    """
    一个用于描述节点各个接口吞吐量的类
    """

    def __init__(self, Node: str):
        self.faceIdMap = dict()
        self.Node = Node

    def appendItem(self, item: ThroughputItem):
        """
        新增一条采样记录
        :param item:
        :return:
        """
        # 如果不是当前节点的统计记录，忽略
        if item.Node != self.Node:
            return
        if item.FaceId not in self.faceIdMap:
            self.faceIdMap[item.FaceId] = NodeFaceThroughput(item.Node, item.FaceId)
        self.faceIdMap[item.FaceId].appendItem(item)

    def getX(self, FaceId: int, throughputType: ThroughputType = ThroughputType.OutData,
             samplingInterval: float = 1.0) -> [float]:
        """
        获取采样时间列表
        :param FaceId:
        :param throughputType:
        :param samplingInterval:
        :return:
        """
        if FaceId not in self.faceIdMap:
            return []
        return self.faceIdMap[FaceId].getX(throughputType, samplingInterval)

    def getY(self, FaceId: int, throughputType: ThroughputType = ThroughputType.OutData,
             throughputTarget: ThroughputTarget = ThroughputTarget.Kilobytes, samplingInterval: float = 1.0):
        """
        获取吞吐量列表
        :param FaceId:                  FaceId
        :param throughputType:          吞吐量类型
        :param throughputTarget:        吞吐量目标
        :param samplingInterval:       采样间隔
        :return:
        """
        if FaceId not in self.faceIdMap:
            return []
        return self.faceIdMap[FaceId].getY(throughputType, throughputTarget, samplingInterval)


class Throughput:
    """
    一个用于解析 ndnsim 吞吐量采样结果的类
    """

    def __init__(self):
        self.nodeMap = dict()

    def getByNode(self, node: str) -> Optional[NodeThroughput]:
        if node not in self.nodeMap:
            return None
        return self.nodeMap[node]

    @staticmethod
    def parse(inputFile: str):
        """
        传入一个 ndnsim 采集的吞吐量结果文件，从中解析出吞吐量
        :param inputFile:
        :return:
        """
        throughput = Throughput()
        with open(inputFile, "r") as f:
            # 首先忽略表头
            # Time	Node	FaceId	FaceDescr	Type	Packets	Kilobytes	PacketRaw	KilobytesRaw
            f.readline()

            # 从第一行开始解析
            line = f.readline()
            while line:
                if line.strip() != "":
                    item = ThroughputItem.parseLine(line)
                    if item:
                        if item.Node not in throughput.nodeMap:
                            throughput.nodeMap[item.Node] = NodeThroughput(item.Node)
                        throughput.nodeMap[item.Node].appendItem(item)
                line = f.readline()
        return throughput


class ThroughputGraph(GraphBase):
    """
    一个用于实现绘制吞吐量图的类
    """

    def __init__(self, throughput: Throughput):
        super().__init__()
        self.throughput = throughput
        self.samplingInterval = 1.0
        self.throughputType = ThroughputType.OutData
        self.throughputTarget = ThroughputTarget.Kilobytes

    def setSamplingInterval(self, samplingInterval: float):
        """
        设置采样间隔 => 如果 samplingInterval = 1.0 表示绘图时，每秒采样一次
        :param samplingInterval:
        :return:
        """
        self.samplingInterval = samplingInterval
        return self

    def setThroughputType(self, throughputType: ThroughputType):
        """
        设置吞吐量类型
        :param throughputType:
        :return:
        """
        self.throughputType = throughputType
        return self

    def setThroughputTarget(self, throughputTarget: ThroughputTarget):
        """
        设置吞吐量目标
        :param throughputTarget:
        :return:
        """
        self.throughputTarget = throughputTarget
        return self

    def plot(self, node: str, faceId: int, *args,
             color: str = "&&",
             linewidth: float = 2, linestyle: str = "dotted", marker: str = "&&",
             markerfacecolor: str = "none", markersize: float = 6,
             **kwargs):
        node = self.throughput.getByNode(node)
        if not node:
            print("not exist node: ", node)
            return
        if "label" not in kwargs:
            kwargs["label"] = node.Node
        x, y = node.getX(faceId, samplingInterval=self.samplingInterval), \
               node.getY(faceId, self.throughputType, self.throughputTarget, self.samplingInterval)
        super().plot(x, y, *args, **kwargs)
        return self

    @staticmethod
    def parse(inputFile: str):
        return ThroughputGraph(Throughput.parse(inputFile))
